"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const react_native_1 = require("react-native");
const utils_1 = require("./utils");
function SectionGrid(props) {
    const { sections, style = {}, spacing = 10, fixed = false, itemDimension = 120, staticDimension, maxDimension, renderItem: originalRenderItem, keyExtractor = null, onLayout = null, additionalRowStyle: externalRowStyle, itemContainerStyle, invertedRow = false, maxItemsPerRow, adjustGridToStyles = false, customSectionList: SectionListComponent = react_native_1.SectionList, onItemsPerRowChange = null, ...restProps } = props;
    const [totalDimension, setTotalDimension] = (0, react_1.useState)(() => {
        let defaultTotalDimension = staticDimension;
        if (!staticDimension) {
            defaultTotalDimension = (0, utils_1.getAdjustedTotalDimensions)({
                totalDimension: react_native_1.Dimensions.get('window').width,
                maxDimension,
                contentContainerStyle: restProps.contentContainerStyle,
                style,
                adjustGridToStyles,
            });
        }
        return defaultTotalDimension;
    });
    const onLocalLayout = (0, react_1.useCallback)((e) => {
        if (!staticDimension) {
            let { width: newTotalDimension } = e.nativeEvent.layout || {};
            newTotalDimension = (0, utils_1.getAdjustedTotalDimensions)({
                totalDimension: newTotalDimension,
                maxDimension,
                contentContainerStyle: restProps.contentContainerStyle,
                style,
                adjustGridToStyles,
            });
            if (totalDimension !== newTotalDimension && newTotalDimension > 0) {
                setTotalDimension(newTotalDimension);
            }
        }
        // call onLayout prop if passed
        if (onLayout) {
            onLayout(e);
        }
    }, [staticDimension, maxDimension, totalDimension, onLayout, adjustGridToStyles, restProps.contentContainerStyle, style]);
    const renderRow = (0, react_1.useCallback)(({ renderItem, rowItems, rowIndex, section, itemsPerRow, rowStyle, separators, isFirstRow, containerStyle, containerFullWidthStyle, }) => {
        // Add spacing below section header
        let additionalRowStyle = {};
        if (isFirstRow) {
            additionalRowStyle = {
                marginTop: spacing,
            };
        }
        const hasFullWidthItem = !!rowItems.find((i) => i._fullWidth);
        return (<react_native_1.View style={[rowStyle, additionalRowStyle, externalRowStyle, hasFullWidthItem ? { flexDirection: 'column', paddingBottom: 0 } : {}]}>
          {rowItems.map((item, index) => {
                const i = invertedRow ? -index + itemsPerRow - 1 : index;
                return (<react_native_1.View key={keyExtractor ? keyExtractor(item, rowIndex * itemsPerRow + i) : `item_${rowIndex * itemsPerRow + i}`} style={[item._fullWidth ? containerFullWidthStyle : containerStyle, itemContainerStyle]}>
                {renderItem({
                        item,
                        index: rowIndex * itemsPerRow + i,
                        section,
                        separators,
                        rowIndex,
                    })}
              </react_native_1.View>);
            })}
        </react_native_1.View>);
    }, [spacing, keyExtractor, externalRowStyle, itemContainerStyle, invertedRow]);
    const { containerDimension, itemsPerRow, fixedSpacing } = (0, react_1.useMemo)(() => (0, utils_1.calculateDimensions)({
        itemDimension,
        staticDimension,
        totalDimension,
        spacing,
        fixed,
        maxItemsPerRow,
    }), [itemDimension, staticDimension, totalDimension, spacing, fixed, maxItemsPerRow]);
    const { containerStyle, containerFullWidthStyle, rowStyle } = (0, react_1.useMemo)(() => (0, utils_1.generateStyles)({
        itemDimension,
        containerDimension,
        spacing,
        fixedSpacing,
        fixed,
        itemsPerRow,
    }), [itemDimension, containerDimension, itemsPerRow, spacing, fixedSpacing, fixed]);
    const groupSectionsFunc = (0, react_1.useCallback)((section) => {
        let chunkedData = (0, utils_1.chunkArray)(section.data, itemsPerRow);
        if (invertedRow) {
            chunkedData = chunkedData.map(($0) => $0.reverse());
        }
        const renderItem = section.renderItem || originalRenderItem;
        return {
            ...section,
            renderItem: ({ item, index, section: s }) => renderRow({
                renderItem,
                rowItems: item,
                rowIndex: index,
                section: s,
                isFirstRow: index === 0,
                itemsPerRow,
                rowStyle,
                containerStyle,
                containerFullWidthStyle,
            }),
            data: chunkedData,
            originalData: section.data,
        };
    }, [itemsPerRow, originalRenderItem, renderRow, rowStyle, containerStyle, containerFullWidthStyle, invertedRow]);
    const groupedSections = sections.map(groupSectionsFunc);
    const localKeyExtractor = (0, react_1.useCallback)((rowItems, index) => {
        if (keyExtractor) {
            return rowItems.map((rowItem, rowItemIndex) => keyExtractor(rowItem, rowItemIndex)).join('_');
        }
        return `row_${index}`;
    }, [keyExtractor]);
    (0, react_1.useEffect)(() => {
        if (onItemsPerRowChange) {
            onItemsPerRowChange(itemsPerRow);
        }
    }, [itemsPerRow, onItemsPerRowChange]);
    return (<SectionListComponent onLayout={onLocalLayout} extraData={totalDimension} sections={groupedSections} keyExtractor={localKeyExtractor} style={style} {...restProps}/>);
}
exports.default = (0, react_1.memo)((0, react_1.forwardRef)(SectionGrid));
